# coding: utf-8
from gat_games.game_engine.engine import *
from gat_games.game_engine.cardgame import *
from gat_games.games.truco import *
from gat_runner.players.gat_player import GATPlayer


class TrucoGATPlayer(GATPlayer):
    def process_response(self, response):
        if response['action'] == 'upcard':
            card = TrucoCard(response['hand_card']['rank'], response['hand_card']['suit'])
            self.upcard(card)
        elif response['action'] == 'truco':
            self.truco()
        elif response['action'] == 'accept_truco':
            self.accept_truco()
        elif response['action'] == 'giveup_truco':
            self.reject_truco()
        else:
            raise InvalidCommandError('Invalid action: %s' % response['action'])

    def upcard(self, card):
        self.game.execute_command(Upcard(self, card=card))

    def truco(self):
        self.game.execute_command(TrucoCommand(self))

    def accept_truco(self):
        self.game.execute_command(AcceptTruco(self))

    def reject_truco(self):
        self.game.execute_command(RejectTruco(self))


Truco.Player = TrucoGATPlayer


# coding: utf-8
import re

import unittest


class InitTests(unittest.TestCase):
    def test_version(self):
        from gat_runner import VERSION
        print(re.match(r'\d[.]\d[.]\d', VERSION))
        self.assertEquals(True, bool(re.match(r'\d[.]\d[.]\d', VERSION)))

    def test_game_import_shortcut(self):
        from gat_runner.players import TrucoGATPlayer
        from gat_runner.players.truco import TrucoGATPlayer



# coding: utf-8
import logging

import random

from gat_games.game_engine.engine import GATLogger
from gat_games.games import *
from gat_runner.languages import GATFileFactory
# Important to update all Game.Player references
from gat_runner.players import *


def play_gat(Game, seed, players, log_level=logging.INFO, player_log=False):
    GATLogger.set_level(log_level)
    for player in players:
        player.start(player_log=player_log)
    if not seed:
        seed = random.randint(1, 9999999)
    game = Game(seed, players)
    game.start()
    game.print_summary()

    for player in players:
        player.stop()
    return game


def play_random_algorithm_vs_random_algorithm(Game, seed, name1=None, name2=None, log_level=logging.INFO):
    players = [Game.RandomStrategy(name=name1), Game.RandomStrategy(name=name2)]
    return play_gat(Game, seed, players, log_level=log_level)


def play_native_algorithm_vs_random_algorithm(Game, seed, custom_strategy, log_level=logging.INFO):
    players = [Game.RandomStrategy(), custom_strategy]
    return play_gat(Game, seed, players, log_level=log_level)


def play_file_against_random_algorithm(Game, seed, filepath, language=None, name1=None, name2=None, log_level=logging.INFO):
    gat_file = GATFileFactory.create(filepath, language=language)
    gat_file.compile()
    command = gat_file.get_runtime_command()
    player1 = Game.Player(command, name=name1)

    player2 = Game.RandomStrategy(name=name2)

    return play_gat(Game, seed, [player1, player2], log_level=log_level)


def play_file_against_file(Game, seed, filepath1, filepath2, language1=None, language2=None, name1=None, name2=None, log_level=logging.INFO):
    gat_file1 = GATFileFactory.create(filepath1, language=language1)
    gat_file1.compile()
    command1 = gat_file1.get_runtime_command()
    player1 = Game.Player(command1, name=name1)

    gat_file2 = GATFileFactory.create(filepath2, language=language2)
    gat_file2.compile()
    command2 = gat_file2.get_runtime_command()
    player2 = Game.Player(command2, name=name2)

    return play_gat(Game, seed, [player1, player2], log_level=log_level)


def play_from_command_line(Game, seed, filepath1='gat-random', filepath2='gat-random', language1=None, language2=None, name1=None, name2=None, log_level=logging.INFO, player_log=False):
    if filepath1 == 'gat-random':
        player1 = Game.RandomStrategy(name=name1)
    else:
        gat_file1 = GATFileFactory.create(filepath1, language=language1)
        gat_file1.compile()
        command1 = gat_file1.get_runtime_command()
        player1 = Game.Player(command1, name=name1)

    if filepath2 == 'gat-random':
        player2 = Game.RandomStrategy(name=name2)
    else:
        gat_file2 = GATFileFactory.create(filepath2, language=language2)
        gat_file2.compile()
        command2 = gat_file2.get_runtime_command()
        player2 = Game.Player(command2, name=name2)

    return play_gat(Game, seed, [player1, player2], log_level=log_level, player_log=player_log)



# coding: utf-8
import unittest

from nose.tools import raises

from gat_runner.languages import *


class GATFileTests(unittest.TestCase):
    def test_get_file_name_and_extensions(self):
        self.assertEquals(('file', 'py'), GATFileFactory.get_filename_and_extension('file.py'))
        self.assertEquals(('file.x', 'py'), GATFileFactory.get_filename_and_extension('file.x.py'))
        self.assertEquals(('file', 'py'), GATFileFactory.get_filename_and_extension('/usr/x/file.py'))

    @raises(ValueError)
    def test_not_supported_language(self):
        GATFileFactory.create('file.invalid_language')

    def test_valid_language(self):
        gatfile = GATFileFactory.create('file.py')
        self.assertEquals(['python', 'file.py'], gatfile.get_runtime_command())


class JavaFileTests(unittest.TestCase):
    def test_main_class_in_the_current_dit(self):
        java = GATFileFactory.create('Class.java')
        main_class = java.get_runtime_command()[3]
        self.assertEquals('Class', main_class)

    def test_main_class_in_a_subdir(self):
        java = GATFileFactory.create('./dir/Class.java')
        main_class = java.get_runtime_command()[3]
        self.assertEquals('Class', main_class)

    def test_main_class_in_a_subdir2(self):
        java = GATFileFactory.create('dir/Class.java')
        main_class = java.get_runtime_command()[3]
        self.assertEquals('Class', main_class)

    def test_main_class_in_a_root_dir(self):
        java = GATFileFactory.create('/dir/Class.java')
        main_class = java.get_runtime_command()[3]
        self.assertEquals('Class', main_class)




# coding: utf-8
import logging
import os
import re
import shutil
import subprocess
import sys


LANGUAGES = {
    'py': 'Python',
    'py2': 'Python 2.7',
    'py3': 'Python 3.3',
    'pypy': 'PyPy',
    'st': 'SmallTalk',
    'rb': 'Ruby',
    'rb2': 'Ruby2',
    'scala': 'Scala',
    'java': 'Java',
    'java6': 'Java6',
    'java7': 'Java7',
    'cs': 'C#',
    'c': 'C',
    'cpp': 'C++',
    'go': 'Go',
    'm': 'Objective-C',
    'js': 'Javascript',
    'lua': 'Lua',
    'gy': 'Groovy',
    'gvy': 'Groovy',
    'groovy': 'Groovy',
    'erl': 'Erlang',
    'hs': 'Haskell',
    'clj': 'Clojure',
    'php': 'PHP',
}


class GATFile(object):
    def __init__(self, filepath, filename, extension, language):
        self.filepath = filepath
        self.filename = filename
        self.extension = extension
        self.language = language

    def get_runtime_command(self):
        return [self.language, self.filepath]

    def get_compilation_command(self):
        pass

    def compile(self):
        compilation_command = self.get_compilation_command()
        if compilation_command:
            try:
                logging.debug('[GAT] %s' % compilation_command)
                # If shell is True, the specified command will be executed through the shell.
                # This can be useful if you are using Python primarily for the enhanced control flow it offers over most system shells and
                # still want convenient access to other shell features such as shell pipes, filename wildcards, environment variable expansion,
                # and expansion of ~ to a user’s home directory.
                # If shell is True, it is recommended to pass args as a string rather than as a sequence.
                result = subprocess.call(compilation_command)
            except Exception as e:
                logging.error('Error on execute the command: %s' % compilation_command)
                raise e
            if result == 0:
                print('%s compilation successful: %s' % (self.language, self.filepath))
            else:
                raise OSError('Compilation error: %s (%s)' % (compilation_command, result))


class PythonFile(GATFile):
    def get_compilation_command(self):
        return ['python', '-m', 'py_compile', self.filepath]

    def get_runtime_command(self):
        return ['python', self.filepath]


class Python3File(GATFile):
    def get_compilation_command(self):
        return ['python3.3', '-m', 'py_compile', self.filepath]

    def get_runtime_command(self):
        return ['python3.3', self.filepath]


class RubyFile(GATFile):
    def get_compilation_command(self):
        return ['ruby', '-c', self.filepath]

    def get_runtime_command(self):
        return ['ruby', self.filepath]


class JavaFile(GATFile):
    def get_classpath(self):
        if sys.platform.startswith('win'):
            return '.;./*;lib/*;bin'
        else:
            return '.:./*:lib/*:bin'

    def get_compilation_command(self):
        if os.path.exists('bin'):
            shutil.rmtree('bin')
        os.mkdir('bin')
        # return ['javac', '-d', 'bin', '-source', '1.6', '-target', '1.6', '-classpath', self.get_classpath(), self.filepath]
        return ['javac', '-d', 'bin', '-classpath', self.get_classpath(), self.filepath]

    def get_runtime_command(self):
        main_class = re.sub('.*/', '', self.filepath).replace('.%s' % self.extension, '')
        return ['java', '-classpath', self.get_classpath(), main_class]


class CFile(GATFile):
    def get_compilation_command(self):
        return ['gcc', self.filepath, '-o', '%s.bin' % self.filepath]

    def get_runtime_command(self):
        return ['%s.bin' % self.filepath]


class CppFile(GATFile):
    def get_compilation_command(self):
        return ['g++', self.filepath, '-o', '%s.bin' % self.filepath]

    def get_runtime_command(self):
        return ['%s.bin' % self.filepath]


class ScalaFile(GATFile):
    def get_classpath(self):
        if sys.platform.startswith('win'):
            return '".;./*;lib/*"'
        else:
            return '".:./*:lib/*"'

    def get_compilation_command(self):
        return ['scalac', '-classpath', self.get_classpath(), self.filepath]

    def get_runtime_command(self):
        main_class = self.filepath.replace('.%s' % self.extension, '')
        return ['java', '-classpath', self.get_classpath(), main_class]


class JavascriptFile(GATFile):
    def get_runtime_command(self):
        return ['phantomjs', self.filepath]


class SmalltalkFile(GATFile):
    def get_runtime_command(self):
        return ['gst', self.filepath]


SUPPORTED_LANGUAGES = {
    'py': PythonFile,
    'py3': Python3File,
    'rb': RubyFile,
    'java': JavaFile,
}

CUSTOM_LANGUAGE_VERSIONS = {
    'py3': 'Python 3.3'
}


class GATFileFactory(object):
    @staticmethod
    def create(filepath, language=None):
        filename, extension = GATFileFactory.get_filename_and_extension(filepath)
        if not language:
            language = extension
        if language not in list(SUPPORTED_LANGUAGES.keys()):
            raise ValueError('Programming language not supported: %s (%s)' % (language, filepath))
        GATFile = SUPPORTED_LANGUAGES[language]
        return GATFile(filepath, filename, extension, language)

    @staticmethod
    def get_filename_and_extension(filepath):
        fullfilename = os.path.splitext(os.path.basename(filepath))
        filename = fullfilename[0]
        extension = fullfilename[1]
        return filename, extension.replace('.', '')



from distutils.version import StrictVersion
import os
from shutil import copy2
from urllib2 import urlopen
import zipfile

import inspect
def s(template, **kwargs):
    '''Usage: s(string, **locals())'''
    if not kwargs:
        frame = inspect.currentframe()
        try:
            kwargs = frame.f_back.f_locals
        finally:
            del frame
        if not kwargs:
            kwargs = globals()
    return template.format(**kwargs)


def colorize(message, color='blue'):
  color_codes = dict(black=30, red=31, green=32, yellow=33, blue=34, magenta=35, cyan=36, white=37)
  code = color_codes.get(color, 34)
  msg = '\033[%(code)sm%(message)s\033[0m' % {'code':code, 'message':message}
  # print(msg)
  return msg


class AutoUpdater(object):
    def __init__(self, dirpath=None):
        if not dirpath:
            self.dirpath = os.path.dirname(os.path.realpath(__file__))
        else:
            self.dirpath = dirpath

    def get_os(self):
        from sys import platform as _platform
        if _platform == "linux" or _platform == "linux2":
            return 'linux'
        elif _platform == "darwin":
            return 'mac'
        elif _platform == "win32":
            return 'windows'

    def download_file(self, filename, url, timeout=10):
        data = urlopen(url, timeout=timeout).read()
        fo = open(s('{self.dirpath}/downloaded_{filename}'), 'w')
        fo.write(data)
        fo.close()

    def update_file(self, filename):
        original_file = s('{self.dirpath}/{filename}')
        backuped_file = s('{self.dirpath}/{filename}.bak')
        downloaded_file = s('{self.dirpath}/downloaded_{filename}')
        if os.path.exists(original_file):
            copy2(original_file, backuped_file)
        copy2(downloaded_file, original_file)

    def install_zip_file(self, filename):
        if filename.endswith('.zip'):
            original_file = s('{self.dirpath}/{filename}')
            zfile = zipfile.ZipFile(original_file)
            for name in zfile.namelist():
                (dirname, filename) = os.path.split(name)
                if dirname and not os.path.exists(dirname):
                    os.mkdir(dirname)
                fd = open(name, 'w')
                fd.write(zfile.read(name))
                fd.close()

    def rollback_updated_file(self, filename):
        backuped_file = s('{self.dirpath}/{filename}.bak')
        if os.path.exists(backuped_file):
            copy2(backuped_file, s('{self.dirpath}/{filename}'))

    def remove_downloaded_file(self, filename):
        downloaded_file = s('{self.dirpath}/downloaded_{filename}')
        if os.path.exists(downloaded_file):
            os.remove(downloaded_file)

    def has_new_version(self, current_version, latest_version_url, timeout=1):
        try:
            latest_version = urlopen(latest_version_url, timeout=1).read()
            result = StrictVersion(latest_version) > StrictVersion(current_version)
            if result:
                print(colorize('New version available (%s). Your version: %s' % (latest_version, current_version), color='blue'))
            else:
                print(colorize('Your version (%s) is up to date.' % current_version, color='white'))
            return result
        except Exception as e:
            print(colorize('Error on verify new version. (%s)' % (str(e),), color='red'))
            return None

    def auto_update(self, files, timeout=10):
        "files = {filename: url}"
        try:
            for filename, url in list(files.items()):
                # print(filename, url)
                self.download_file(filename, url, timeout=timeout)
        except Exception as e:
            print(colorize('Auto update failed because a download error. (%s)' % str(e), color='red'))
            return

        try:
            for filename, url in list(files.items()):
                # print(filename)
                self.update_file(filename)
                self.install_zip_file(filename)
            print(colorize('Update successfully', color='green'))
            for filename, url in list(files.items()):
                self.remove_downloaded_file(filename)
        except Exception as e:
            import sys
            raise e, None, sys.exc_info()[2]
            for filename, url in list(files.items()):
                self.rollback_updated_file(filename)
            print(colorize('Update error. (%s)' % (str(e)), color='red'))




# coding: utf-8
import os
import tempfile
import unittest
from nose.tools import raises

import gat_games.game_engine.gat_json as json
from gat_games.game_engine.engine import *
from gat_runner.players.gat_player import *


class StubGATPlayer(GATPlayer):
    def play(self, context, **kwargs):
        self.worked = False
        self.response = super(StubGATPlayer, self).play(context, **kwargs)
        self.worked = True
        return self.response


class GATPlayerTests(unittest.TestCase):
    def test_gat_player_must_be_serializable(self):
        result = json.dumps(StubGATPlayer(['python',  'x']))
        self.assertEquals('{}', result)

    def execute_algorithm(self, code, response):
        f, filepath = tempfile.mkstemp(text=True)
        f = open(filepath, 'w')
        f.write(code)
        f.close()
        player = StubGATPlayer(['python',  filepath], timeout=1)
        try:
            player.start()
            player.play({}) # send message to the algorithm
            self.assertEquals(True, player.worked)
            self.assertEquals(response, player.response)
        finally:
            player.stop()
            os.remove(filepath)

    def test_correct_algorithm(self):
        code = '''
print('Running Correct Algorithm')
import socket
import sys
print('[1] Args: ' + str(sys.argv))
port = int(sys.argv[1])

sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.bind(('localhost', port))
sock.listen(1)
conn, addr = sock.accept()

print('[1] Algorithm waiting for message')
message = conn.recv(8192)

print('[1] Algorithm sending message')
conn.sendall(message)
print('[1] Message sent')
'''
        self.execute_algorithm(code, {u'action': u'play', u'context': {}})

    @raises(AlgorithmTimeoutError)
    def test_slow_algorithm(self):
        code = '''
print('Running Slow Algorithm')
import socket
import sys
print('[2] Args: ' + str(sys.argv))
port = int(sys.argv[1])

sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.bind(('localhost', port))
sock.listen(1)
conn, addr = sock.accept()

import time
time.sleep(2)
'''
        self.execute_algorithm(code, None)

    @raises(AlgorithmError)
    def test_bugged_algorithm(self):
        code = '''
print('Running Bugged Algorithm')
import socket
import sys
print('[3] Args: ' + str(sys.argv))
port = int(sys.argv[1])

sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
sock.bind(('localhost', port))
sock.listen(1)
conn, addr = sock.accept()

print('[3] Algorithm waiting for message')
message = conn.recv(8192)

print('[3] Algorithm sending message')
error = '{"error":"exception message"}'
if sys.version_info[0] == 2:
    conn.sendall(error)
else:
    conn.sendall(bytes(error, 'utf-8'))
print('[3] Message sent')
'''
        self.execute_algorithm(code, None)

    @raises(AlgorithmInitializationError)
    def test_bugged_algorithm_in_the_initialization(self):
        code = '''
print('Running Bugged Algorithm')
raise Exception('ops')
'''
        self.execute_algorithm(code, None)



if __name__ == "__main__":
    import os
    import sys
    sys.path.append(os.getcwd())

    import logging

    from gat_games.games.truco import *

    truco = Truco(1, [RandomTrucoPlayer('Player1'), RandomTrucoPlayer('Player2')])

    FORMAT = '%(message)s'
    logging.basicConfig(format=FORMAT)
    truco.logger.setLevel(logging.INFO)
    # truco.logger.setLevel(logging.DEBUG)

    report = truco.play()
    truco.logger.debug(report)




# coding: utf-8
# http://rosettacode.org/wiki/Hello_world/Standard_error
# http://en.wikipedia.org/wiki/Inter-process_communication
# http://docs.python.org/2/library/subprocess.html
# http://docs.python.org/3/library/subprocess.html
# http://docs.python.org/2/library/subprocess.html#subprocess.Popen
# http://jimmyg.org/blog/2009/working-with-python-subprocess.html
# http://www.decalage.info/python/ruby_bridge
import logging
import os
import random
from subprocess import Popen
import socket
import sys
import threading
import time

import gat_games.game_engine.gat_json as json
from gat_games.game_engine.engine import Player, GATLogger, AlgorithmError, AlgorithmInitializationError, AlgorithmTimeoutError


def run_with_time_limit(timeout, func, args=(), kwargs={}):
    class FuncThread(threading.Thread):
        def __init__(self):
            super(FuncThread, self).__init__()
            self.result = None

        def run(self):
            self.result = func(*args, **kwargs)

        def stop(self):
            if self.isAlive():
                if hasattr(self, '_Thread__stop'):
                    self._Thread__stop()

    thread = FuncThread()
    thread.start()
    thread.join(timeout)
    if thread.isAlive():
        thread.stop()
        raise AlgorithmTimeoutError('Algorithm timeout error')
    else:
        return thread.result


class GATPlayer(Player):
    DELIMITER = '\n'
    CONNECT_TIMEOUT = 10 # in seconds

    def __init__(self, runtime_command, timeout=1*60, **kwargs):
        super(GATPlayer, self).__init__(**kwargs)
        self.runtime_command = runtime_command
        self.timeout = timeout
        self.algorithm = None
        self.context = None
        self.sock = None
        self.file = None

    def __getstate__(self):
        odict = self.__dict__.copy()
        odict.pop('algorithm', None)
        odict.pop('context', None)
        odict.pop('sock', None)
        odict.pop('conn', None)
        return odict

    def get_valid_port(self, host='localhost'):
        for i in list(range(0, 10)): # avoid infinite loop
            # http://www.iana.org/assignments/service-names-port-numbers/service-names-port-numbers.xml
            port = random.randint(1025, 65535)
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            try:
                result = sock.connect_ex((host, port))
                if result != 0:
                    return port
            finally:
                sock.close()
                time.sleep(0.5)
        return 0

    def start_algorithm_safely(self, runtime_command, player_log=True):
        env = {}
        env_publich_vars = ['PWD', 'PATH', 'HOME', 'TERM', 'SHELL',
            'LOGNAME', 'USER', 'USERNAME', 'SUDO_USER',
            '_',
            'RUBYLIB', 'GEM_PATH', 'GEM_HOME', 'rvm_path', 'rvm_bin_path']
        for var in env_publich_vars:
            value = os.getenv(var, None)
            if value:
                env[var] = value
        # server socket
        if player_log:
            stdout = open('%s.log' % str(self), 'w')
        else:
            stdout = None
        # custom env and stdout/stderr for security
        self.algorithm = Popen(runtime_command, cwd=None, env=env, stdout=stdout, stderr=stdout)
        # self.algorithm.wait() # does not work properly
        time.sleep(1) # giving time to the server start properly

    def start(self, host='localhost', port=None, player_log=False):
        try:
            self.log('Starting algorithm: %s' % self.runtime_command, logging.DEBUG)
            if not port:
                port = self.get_valid_port(host=host)
            self.runtime_command.append(str(port))
            self.runtime_command.append(str(GATLogger.get_log_level()))

            self.start_algorithm_safely(self.runtime_command, player_log=player_log)

            self.log('Algorithm started', logging.DEBUG)
            self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM) # client socket
            self.sock.settimeout(self.CONNECT_TIMEOUT) # (connection timeout only)
            self.sock.connect((host, port))

            # makefile must use blocking mode
            self.sock.settimeout(None)
            self.sock.setblocking(1)

            self.file = self.sock.makefile("rb") # buffered
            self.log('Algorithm connected', logging.DEBUG)
        except Exception as e:
            self.log(self.runtime_command, logging.ERROR)
            self.log(str(e), logging.ERROR)
            self.stop()
            raise AlgorithmInitializationError(str(e))

    def stop(self):
        if self.sock:
            self.sock.close()
            self.log('Socket closed', logging.DEBUG)
        if self.algorithm:
            self.algorithm.kill()
            self.log('Process killed', logging.DEBUG)

    def send_message_to_algorithm(self, message):
        self.log('>> Sending message to algorithm:\n%s' % message, logging.DEBUG)
        message = '%s%s' % (message, self.DELIMITER)
        if sys.version_info[0] == 2:
            self.sock.sendall(message)
        else:
            self.sock.sendall(bytes(message, 'utf-8'))

    def receive_message_from_algorithm(self):
        self.log('<< Receiving response from algorithm', logging.DEBUG)
        try:
            message = self.file.readline()
            message = message.decode("utf-8") # Python 3
            if message:
                message = message.replace(self.DELIMITER, '')
                message = message.strip()
            self.log('Response: %s' % message, logging.DEBUG)
            return message
        except socket.timeout: # blocking socket will not raise timeout here
            self.log('Timeout %s' % str(self), logging.ERROR)
            raise AlgorithmTimeoutError('Algorithm timeout error: %s' % str(self))

    def delegate_decision_to_the_algorithm(self, message):
        message = json.dumps(message)
        self.send_message_to_algorithm(message)
        response = run_with_time_limit(self.timeout, self.receive_message_from_algorithm)
        if response:
            response = json.loads(response)
            error = response.get('error', None)
            if error:
                raise AlgorithmError('Algorithm error: %s' % error)
            else:
                return self.process_response(response)

    def process_response(self, response):
        return response

    def play(self, context, **kwargs):
        message = {'action':kwargs.get('action', 'play'), 'context':context}
        return self.delegate_decision_to_the_algorithm(message)




# coding: utf-8
import os
import unittest

from nose.tools import raises

from gat_games import *
from gat_runner.runner import *


class PlayGatTests(unittest.TestCase):
    def test_play_with_native_python_players(self):
        game = play_gat(Truco, 1, [RandomTrucoPlayer(), RandomTrucoPlayer()])
        report = game.summary()
        self.assertEquals(None, report['error'])
        self.assertEquals(True, report['scoreboard'][report['winners'][0]] >= 12)


class PlayNativePlayerAgainstRandomAlgorithmTests(unittest.TestCase):
    def test_play_native_algorithm_vs_random_algorithm(self):
        play_native_algorithm_vs_random_algorithm(Truco, 1, RandomTrucoPlayer())


class PlayFileAgainstRandomAlgorithmTests(unittest.TestCase):
    def test_play_file_against_random_algorithm(self):
        full_path = os.path.realpath(__file__)
        dirpath, filename = os.path.split(full_path)
        filepath = os.path.join(dirpath, 'sample_truco_algorithm.py')
        print(filepath)
        play_file_against_random_algorithm(Truco, 1, filepath)


class PlayFileAgainstFileTests(unittest.TestCase):
    def test_play_file_against_file(self):
        full_path = os.path.realpath(__file__)
        dirpath, filename = os.path.split(full_path)
        filepath = os.path.join(dirpath, 'sample_truco_algorithm.py')
        print(filepath)
        play_file_against_file(Truco, 1, filepath, filepath)




#from distutils.core import setup
from setuptools import setup, find_packages

VERSION = "0.0.2"
PROJECT_NAME = 'gat-runner'

tests_require = [
]

install_requires = [
  'six==1.3.0',
  'simplejson==3.1.3',
  'gat-games==0.0.2',
]

setup(name='%s' % PROJECT_NAME,
      url='https://github.com/gatournament/%s' % PROJECT_NAME,
      author="gatournament",
      author_email='info@gatournament.com',
      keywords='gat game algorithms tournament',
      description='GAT Runner',
      license='MIT',
      classifiers=[
          'Operating System :: OS Independent',
          'Topic :: Software Development'
      ],

      version='%s' % VERSION,
      install_requires=install_requires,
      tests_require=tests_require,
      # test_suite='runtests.runtests',
      # extras_require={'test': tests_require},

      packages=find_packages(),
)



import os
import socket
import sys
sys.path.append(os.getcwd())

from functools import update_wrapper

import gat_games.game_engine.gat_json as json

print('\n')
print('Running Sample Truco Algorithm')

# http://docs.python.org/2/library/argparse.html
# http://lucumr.pocoo.org/2012/6/26/disconnects-are-good-for-you/
port = int(sys.argv[1]) if len(sys.argv) > 1 else 58888
sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
print('Binding %s' % str(('localhost', port)))
sock.bind(('localhost', port))
print('Listening')
sock.listen(1)
conn, addr = sock.accept()
print('Client connected.')

def send_message(message):
    if sys.version_info[0] == 2:
        conn.sendall(message)
    else:
        conn.sendall(bytes(message, 'utf-8'))

while True:
    try:
        # print('\nWaiting for request')
        message = conn.recv(8192) # 2**13
        if not message or message == 'stop':
            break
        # print('\nRequest received: %s' % json.loads(message))
        message = json.loads(message)

        # print('Context %s' % message['context'])

        response = {'action': 'upcard', 'hand_card': message['context']['hand']['cards'][0]}

        # print('\nAnswer: %s' % response)
        response = json.dumps(response) + '\n'
        send_message(response)
        # print('\nSent' % response)
    except Exception as e:
        response = json.dumps({'error': str(e)})
        send_message(response)

conn.close()
